<?php
/*
Plugin Name: Raffle Plugin
Description: A simple raffle plugin that sends a unique alphanumeric code to users via email. I also included admin settings.
Version: 1.1
Author: SB
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

// Activation hook to create the database table and default settings
register_activation_hook(__FILE__, 'raffle_create_table');
function raffle_create_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'raffle_entries';
    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        email varchar(100) NOT NULL,
        code varchar(6) NOT NULL,
        PRIMARY KEY (id),
        UNIQUE (code)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);

    // Set default raffle status and shortcode settings
    if (get_option('raffle_status') === false) {
        update_option('raffle_status', 'closed');
    }
    if (get_option('raffle_winner_shortcode') === false) {
        update_option('raffle_winner_shortcode', 'on');
    }
    if (get_option('raffle_gdpr_shortcode') === false) {
        update_option('raffle_gdpr_shortcode', 'on');
    }
}

// Enqueue custom CSS
function raffle_enqueue_styles() {
    wp_enqueue_style('raffle-plugin-styles', plugins_url('/css/raffle-plugin-styles.css', __FILE__));
}
add_action('wp_enqueue_scripts', 'raffle_enqueue_styles');

// Shortcode for the email form
function raffle_email_form() {
    $raffle_status = get_option('raffle_status');
    $message = isset($_POST['raffle_message']) ? esc_html($_POST['raffle_message']) : '';
    $error = isset($_POST['raffle_error']) ? esc_html($_POST['raffle_error']) : '';
    $website_name = get_bloginfo('name');
    
    ob_start();
    if ($raffle_status === 'open') {
        ?>
        <form method="post" action="" class="raffle-form">
            <input type="email" name="raffle_email" required placeholder="Enter your email" class="raffle-input same-width-input">
            <input type="submit" name="submit_raffle" value="Enter Raffle" class="raffle-button">
            <?php if ($message) : ?>
                <p class="raffle-message"><?php echo $message; ?><br><?php echo $website_name; ?></p>
            <?php endif; ?>
            <?php if ($error) : ?>
                <p class="raffle-error"><?php echo $error; ?></p>
            <?php endif; ?>
        </form>
        <?php
    } else {
        echo '<p class="raffle-closed">The raffle is currently closed. Please check back later.</p>';
    }
    return ob_get_clean();
}
add_shortcode('raffle_form', 'raffle_email_form');

// Handle form submission
function raffle_handle_form_submission() {
    if (isset($_POST['submit_raffle']) && isset($_POST['raffle_email']) && is_email($_POST['raffle_email'])) {
        $email = sanitize_email($_POST['raffle_email']);

        global $wpdb;
        $table_name = $wpdb->prefix . 'raffle_entries';

        // Check if email already exists
        $existing_email = $wpdb->get_var($wpdb->prepare("SELECT email FROM $table_name WHERE email = %s", $email));

        if ($existing_email) {
            $_POST['raffle_error'] = 'This email address has already been used.';
        } else {
            $code = raffle_generate_unique_code();

            // Insert into the database
            $wpdb->insert(
                $table_name,
                array(
                    'email' => $email,
                    'code' => $code
                )
            );

            // Send email
            $subject = 'Your Raffle Code';
            $website_name = get_bloginfo('name');
            $message = 'Thank you for entering the raffle on ' . $website_name . '. Your code is: ' . $code;
            wp_mail($email, $subject, $message);

            $_POST['raffle_message'] = 'Thank you for entering the raffle. Check your email for your code.';
        }
    }
}
add_action('wp', 'raffle_handle_form_submission');

// Generate unique random code
function raffle_generate_unique_code() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'raffle_entries';
    
    do {
        $code = strtoupper(wp_generate_password(6, false, false));
        $existing_code = $wpdb->get_var($wpdb->prepare("SELECT code FROM $table_name WHERE code = %s", $code));
    } while ($existing_code);

    return $code;
}

// Admin interface to manage raffle entries
add_action('admin_menu', 'raffle_admin_menu');
function raffle_admin_menu() {
    add_menu_page(
        'Raffle Entries',
        'Raffle Entries',
        'manage_options',
        'raffle-entries',
        'raffle_entries_page'
    );

    add_submenu_page(
        'raffle-entries',
        'Raffle Settings',
        'Settings',
        'manage_options',
        'raffle-settings',
        'raffle_settings_page'
    );
}

function raffle_entries_page() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'raffle_entries';
    $entries = $wpdb->get_results("SELECT * FROM $table_name");
    $random_entry = isset($_GET['random_entry']) ? intval($_GET['random_entry']) : 0;
    $random_code = '';

    if ($random_entry) {
        $random_code = $wpdb->get_var($wpdb->prepare("SELECT code FROM $table_name WHERE id = %d", $random_entry));
    }

    ?>
    <div class="wrap">
        <h1>Raffle Entries</h1>
        <div style="margin-bottom: 20px;">
            <form method="post" action="" style="display: inline;">
                <input type="submit" name="export_csv" class="button button-primary" value="Export CSV">
            </form>
            <form method="post" action="" style="display: inline;">
                <input type="submit" name="pick_random" class="button button-secondary" value="Pick Random Code">
            </form>
        </div>
        <?php if ($random_code) : ?>
            <div style="margin-bottom: 20px; padding: 10px; background-color: #f1f1f1; border-left: 4px solid #0073aa;">
                <h2>Randomly Selected Code: <?php echo esc_html($random_code); ?></h2>
            </div>
        <?php endif; ?>
        <table class="wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <th>ID</th>
                    <th>Email</th>
                    <th>Code</th>
                    <th>Action</th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ($entries as $entry): ?>
                <tr>
                    <td><?php echo esc_html($entry->id); ?></td>
                    <td><?php echo esc_html($entry->email); ?></td>
                    <td><?php echo esc_html($entry->code); ?></td>
                    <td><a href="<?php echo admin_url('admin.php?page=raffle-entries&delete=' . $entry->id); ?>" class="button button-link-delete">Delete</a></td>
                </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
    </div>
    <?php
}

function raffle_settings_page() {
    if (isset($_POST['raffle_status']) || isset($_POST['raffle_winner_shortcode']) || isset($_POST['raffle_gdpr_shortcode'])) {
        update_option('raffle_status', sanitize_text_field($_POST['raffle_status']));
        update_option('raffle_winner_shortcode', isset($_POST['raffle_winner_shortcode']) ? 'on' : 'off');
        update_option('raffle_gdpr_shortcode', isset($_POST['raffle_gdpr_shortcode']) ? 'on' : 'off');
    }

    $raffle_status = get_option('raffle_status', 'closed');
    $raffle_winner_shortcode = get_option('raffle_winner_shortcode', 'on');
    $raffle_gdpr_shortcode = get_option('raffle_gdpr_shortcode', 'on');
    ?>
    <div class="wrap">
        <h1>Raffle Settings</h1>
        <form method="post" action="">
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Raffle Status</th>
                    <td>
                        <select name="raffle_status">
                            <option value="open" <?php selected($raffle_status, 'open'); ?>>Open</option>
                            <option value="closed" <?php selected($raffle_status, 'closed'); ?>>Closed</option>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Display Winner Shortcode</th>
                    <td>
                        <input type="checkbox" name="raffle_winner_shortcode" <?php checked($raffle_winner_shortcode, 'on'); ?>>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Display GDPR Shortcode</th>
                    <td>
                        <input type="checkbox" name="raffle_gdpr_shortcode" <?php checked($raffle_gdpr_shortcode, 'on'); ?>>
                    </td>
                </tr>
            </table>
            <?php submit_button('Save Changes'); ?>
        </form>
    </div>
    <?php
}

// Handle entry deletion
add_action('admin_init', 'raffle_handle_delete');
function raffle_handle_delete() {
    if (isset($_GET['delete']) && current_user_can('manage_options')) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'raffle_entries';
        $id = intval($_GET['delete']);

        $wpdb->delete($table_name, array('id' => $id));

        wp_redirect(admin_url('admin.php?page=raffle-entries'));
        exit;
    }
}

// Handle CSV export
add_action('admin_init', 'raffle_handle_csv_export');
function raffle_handle_csv_export() {
    if (isset($_POST['export_csv']) && current_user_can('manage_options')) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'raffle_entries';
        $entries = $wpdb->get_results("SELECT * FROM $table_name", ARRAY_A);

        if (!empty($entries)) {
            $filename = 'raffle_entries_' . date('Y-m-d') . '.csv';

            header('Content-Type: text/csv');
            header('Content-Disposition: attachment;filename=' . $filename);

            $output = fopen('php://output', 'w');
            fputcsv($output, array('ID', 'Email', 'Code'));

            foreach ($entries as $entry) {
                fputcsv($output, $entry);
            }

            fclose($output);
            exit;
        }
    }
}

// Handle picking a random code
add_action('admin_init', 'raffle_handle_random_pick');
function raffle_handle_random_pick() {
    if (isset($_POST['pick_random']) && current_user_can('manage_options')) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'raffle_entries';
        $random_entry = $wpdb->get_var("SELECT id FROM $table_name ORDER BY RAND() LIMIT 1");

        if ($random_entry) {
            $winner = $wpdb->get_row($wpdb->prepare("SELECT email, code FROM $table_name WHERE id = %d", $random_entry));
            $random_code = $winner->code;
            $winner_email = $winner->email;

            set_transient('raffle_winner_code', $random_code, 12 * HOUR_IN_SECONDS); // Store the code for 12 hours
            wp_redirect(admin_url('admin.php?page=raffle-entries&random_entry=' . $random_entry));
            exit;
        }
    }
}

// Shortcode to display the random code
function display_random_code() {
    $raffle_winner_shortcode = get_option('raffle_winner_shortcode', 'on');
    if ($raffle_winner_shortcode === 'off') {
        return '';
    }

    $random_code = get_transient('raffle_winner_code');
    $raffle_status = get_option('raffle_status');
    if ($raffle_status === 'closed') {
        return '<div style="text-align: center;">The raffle is currently closed. Please check back later.</div>';
    } elseif ($random_code) {
        return '<div style="text-align: center;">The winning code is: <strong>' . esc_html($random_code) . '</strong></div>';
    } else {
        return '<div style="text-align: center;">No winner has been selected yet.</div>';
    }
}
add_shortcode('display_winner', 'display_random_code');

// Shortcode for GDPR compliance form
function raffle_gdpr_form() {
    $raffle_gdpr_shortcode = get_option('raffle_gdpr_shortcode', 'on');
    if ($raffle_gdpr_shortcode === 'off') {
        return '';
    }

    $gdpr_message = isset($_POST['gdpr_message']) ? esc_html($_POST['gdpr_message']) : '';
    ob_start(); ?>
    <div style="text-align: center; max-width: 600px; margin: 0 auto;">
        <p>This form allows you to delete your data from our raffle entries in compliance with GDPR.</p>
        <form method="post" action="">
            <input type="email" name="gdpr_email" required placeholder="Enter your email to delete your data" class="raffle-input same-width-input">
            <input type="submit" name="delete_data" value="Delete My Data" class="raffle-button">
        </form>
        <?php if ($gdpr_message) : ?>
            <p class="raffle-message"><?php echo $gdpr_message; ?></p>
        <?php endif; ?>
    </div>
    <?php
    return ob_get_clean();
}
add_shortcode('raffle_gdpr', 'raffle_gdpr_form');

// Handle GDPR form submission
function raffle_handle_gdpr_form_submission() {
    if (isset($_POST['delete_data']) && isset($_POST['gdpr_email']) && is_email($_POST['gdpr_email'])) {
        $email = sanitize_email($_POST['gdpr_email']);

        global $wpdb;
        $table_name = $wpdb->prefix . 'raffle_entries';

        $wpdb->delete($table_name, array('email' => $email));

        $_POST['gdpr_message'] = 'Your data has been deleted.';
    }
}
add_action('wp', 'raffle_handle_gdpr_form_submission');

// Custom CSS
function raffle_custom_styles() {
    ?>
    <style>
        .raffle-form, .raffle-message, .raffle-error, .raffle-closed {
            text-align: center;
        }
        .raffle-input {
            margin: 10px auto;
            padding: 10px;
            width: 70%;
            max-width: 300px;
        }
        .same-width-input {
            width: 70%;
            max-width: 300px;
        }
        .raffle-button {
            margin: 10px auto;
            padding: 10px 20px;
            background-color: #0073aa; /* Change this to your desired button color */
            color: #fff;
            border: none;
            cursor: pointer;
        }
    </style>
    <?php
}
add_action('wp_head', 'raffle_custom_styles');
